/* Demonstrates SHFormatDrive from Shell32 */
#define WIN32_LEAN_AND_MEAN
#define STRICT
#ifdef _MSC_VER
#   pragma warning(disable:4201 4214 4514)
#endif
#include <windows.h>
#include <windowsx.h>
#undef  UNREFERENCED_PARAMETER
#define UNREFERENCED_PARAMETER(P)   ((void)(P))
#include "resource.h"
#ifdef _MSC_VER
#   pragma warning(default:4201 4214)
#endif

#include "Format.h"

static BOOL g_bIsNT;
static INT g_iCapacity, g_iType; 

static BOOL MPF_OnOk( HWND );
static VOID MPF_OnInitDialog( HWND );
static VOID ERR( INT );

LRESULT CALLBACK APP_DlgProc( HWND, UINT, WPARAM, LPARAM );


INT APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevious, 
   LPSTR lpsz, INT iCmd )
{
    UNREFERENCED_PARAMETER(hPrevious);
    UNREFERENCED_PARAMETER(lpsz);
    UNREFERENCED_PARAMETER(iCmd);

    return DialogBox( hInstance, "DLG_MAIN", 
        NULL, (DLGPROC)APP_DlgProc );
}


LRESULT CALLBACK APP_DlgProc( HWND hDlg, UINT uiMsg, 
     WPARAM wParam, LPARAM lParam )
{
    UNREFERENCED_PARAMETER(lParam);
   switch( uiMsg )
   {
       case WM_INITDIALOG:
          MPF_OnInitDialog( hDlg );
          break;
     
       case WM_COMMAND:
          switch( wParam )		    
          {
              case IDOK:
                 MPF_OnOk( hDlg );
                 return 1;

              case IDCANCEL: 
                 EndDialog( hDlg, FALSE );
                 return 0;

              case IDC_360:
              case IDC_720:
              case IDC_DEFAULT:
                 g_iCapacity = LOWORD(wParam);
                 break;

              case IDC_SYSTEM:
              case IDC_FULL:
              case IDC_QUICK:
                 g_iType = LOWORD(wParam);
                 break;
           }
           break;
     }
     return 0;
}


BOOL MPF_OnOk( HWND hDlg )
{
   HWND hwndCbo;
   INT irc, iDrive, iCapacity=0, iType=0;

   hwndCbo = GetDlgItem(hDlg, IDC_DRIVE);
   iDrive = ComboBox_GetCurSel( hwndCbo );
   if( g_iCapacity==IDC_360 )
      iCapacity = FORMAT_CAP_360;
   if( g_iCapacity==IDC_720 )
      iCapacity = FORMAT_CAP_720;
   if( g_iType==IDC_QUICK )
      iType = QUICK_FORMAT;
   if( g_iType==IDC_FULL )
      iType = FULL_FORMAT;
   if( g_iType==IDC_SYSTEM )
      iType = SYSTEM_FORMAT;

   irc = FormatDrive( hDlg, iDrive, iCapacity, iType );
   ERR(irc);
   return TRUE;
}

VOID MPF_OnInitDialog( HWND hDlg )
{
   HWND hwndCbo;
   OSVERSIONINFO os;
   os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
   GetVersionEx( &os );
   g_bIsNT = (os.dwPlatformId==VER_PLATFORM_WIN32_NT);

   g_iType = (g_bIsNT ?IDC_FULL :IDC_QUICK);
   g_iCapacity = IDC_DEFAULT;
   CheckRadioButton( hDlg, IDC_DEFAULT, IDC_720, g_iCapacity );
   CheckRadioButton( hDlg, IDC_QUICK, IDC_SYSTEM, g_iType );

   if( g_bIsNT )
   {
      EnableWindow( GetDlgItem(hDlg, IDC_SYSTEM), FALSE );
      EnableWindow( GetDlgItem(hDlg, IDC_360), FALSE );
      EnableWindow( GetDlgItem(hDlg, IDC_720), FALSE );
   }

   hwndCbo = GetDlgItem(hDlg, IDC_DRIVE);
#ifdef __BORLANDC__
#pragma warn -eff
#endif
   ComboBox_AddString(hwndCbo, " A:");
   ComboBox_AddString(hwndCbo, " B:");
   ComboBox_AddString(hwndCbo, " C:");
   ComboBox_AddString(hwndCbo, " D:");
   ComboBox_AddString(hwndCbo, " E:");
   ComboBox_SetCurSel(hwndCbo, 0);
#ifdef __BORLANDC__
#pragma warn .eff
#endif
}

VOID ERR( INT iErrCode )
{
  CHAR szBuf[MAX_PATH], szMsg[40];
  switch( iErrCode )
  {
     case ERR_IMPOSSIBLE:
       lstrcpy( szMsg, "Unable to format." );
       break;
     case ERR_NOTFLOPPY:
       lstrcpy( szMsg, "Not a floppy drive." );
       break;
     case ERR_WRONGPARAMS:
       lstrcpy( szMsg, "Unacceptable params." );
       break;
     case ERR_NONE:
       lstrcpy( szMsg, "Done." );
       break;
     case ERR_CANCELED:
       lstrcpy( szMsg, "Aborted." );
       break;
     default:
       lstrcpy( szMsg, "Unknown error." );
  }
  wsprintf( szBuf, "%s (return code %d)", szMsg, iErrCode );
  MessageBox( GetFocus(), szBuf, "Error", MB_ICONINFORMATION );
}

