// dirsize.c

#include <stdio.h>
#include <direct.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>

#ifdef _WIN32

    #define WIN32_LEAN_AND_MEAN

    #include <windows.h>
    #include <io.h>

    #define findFirstFile( x, y, z ) _findfirst(x,z)
    #define findNextFile( x, y )     _findnext(x,y)
    #define closeSearch( x )         _findclose(x)
    #define IS_FIND_ERROR( x )       (-1 == (x))

    typedef struct _finddata_t FILE_DESCR;

#else 

    #include <dos.h>

    #define findFirstFile( x, y, z ) _dos_findfirst(x,y,z)
    #define findNextFile( x, y )     _dos_findnext(y)
    #define closeSearch( x )
    #define IS_FIND_ERROR( x )       (0 != (x))

    #define MAX_PATH _MAX_PATH

    typedef struct _find_t FILE_DESCR;
    typedef unsigned long  DWORD;
    typedef unsigned short WORD;
    typedef int            BOOL;

#endif // _WIN32


#define SEARCH_ATTRS ( _A_RDONLY | _A_HIDDEN | _A_SYSTEM | \
                       _A_VOLID  | _A_SUBDIR | _A_ARCH )

static void getDirectorySize( 
    char *pszPath, DWORD *pdwSIze, DWORD *pdwUsed );

static DWORD g_dwBytesPerCluster = 0;


static DWORD getClusterSize( char *pszDir ) 
{
    BOOL bPathOK;
    DWORD dwBytesPerSector = 0;
    DWORD dwSectorsPerCluster = 0;
    
    char szRoot[ MAX_PATH ];
    if ( !_fullpath( szRoot, pszDir, MAX_PATH ) ) {
        fprintf( stderr, "Can't get full path of %s\n", pszDir );
        exit( 1 );
    }
    szRoot[ 3 ] = 0;
    
    {
#ifdef _WIN32
        DWORD dwNumberOfFreeClusters;
        DWORD dwTotalNumberOfClusters;
        bPathOK = GetDiskFreeSpace( szRoot, &dwSectorsPerCluster, 
            &dwBytesPerSector, &dwNumberOfFreeClusters, 
            &dwTotalNumberOfClusters );
#else
        char chDisk = toupper( szRoot[ 0 ] ) - '@';
        struct diskfree_t drvinfo;
        bPathOK = (0 == _dos_getdiskfree( chDisk, &drvinfo ));
        dwBytesPerSector    = (DWORD) drvinfo.bytes_per_sector;
        dwSectorsPerCluster = (DWORD) drvinfo.sectors_per_cluster;
#endif
    }
    
    if ( !bPathOK ) {
        fprintf( stderr, "Can't get sector size on %s\n", szRoot );
        exit( 1 );
    }

#ifdef _DEBUG
    printf( "dwBytesPerSector = %lu\n"
            "dwSectorsPerCluster = %lu\n", 
        dwBytesPerSector, dwSectorsPerCluster );
#endif

    return dwBytesPerSector * dwSectorsPerCluster;
}


static DWORD getBytesUsed( DWORD dwSize ) 
{
    DWORD dwClustersUsed;
    assert( 0 < g_dwBytesPerCluster );
    dwClustersUsed = 
        (dwSize + g_dwBytesPerCluster - 1) / g_dwBytesPerCluster;
    return dwClustersUsed * g_dwBytesPerCluster;
}


static void getFileSize( 
    char *pszPath, FILE_DESCR *pFileDescr, 
    DWORD *pdwSize, DWORD *pdwUsed )
{
    char szSubDir[ MAX_PATH ];
    
    if ( pFileDescr->name[ 0 ] != '.' ) {
        if ( pFileDescr->attrib & _A_SUBDIR ) {
            strcpy( szSubDir, pszPath );
            szSubDir[ strlen( pszPath ) - 3 ] = 0;
            strcat( szSubDir, pFileDescr->name );
            getDirectorySize( szSubDir, pdwSize, pdwUsed );
        } else {
            *pdwSize += pFileDescr->size;
            *pdwUsed += getBytesUsed( pFileDescr->size );
        }
    }
}


static void getDirectorySize( 
    char *pszPath, DWORD *pdwSize, DWORD *pdwUsed ) 
{
    FILE_DESCR fileDescr;
    long hf;
    
    strcat( pszPath, "\\*.*" );
    hf = (long) findFirstFile( pszPath, SEARCH_ATTRS, &fileDescr );
    if ( !IS_FIND_ERROR( hf ) ) {
        getFileSize( pszPath, &fileDescr, pdwSize, pdwUsed );
        while ( 0 == findNextFile( hf, &fileDescr ) ) {
            getFileSize( pszPath, &fileDescr, pdwSize, pdwUsed );
        }
        closeSearch( hf );
    }
}


static BOOL isDirectory( char *pszPath ) 
{
#ifdef _WIN32
    DWORD dwRet = GetFileAttributes( pszPath );
    return (dwRet != 0xFFFFFFFF) && 
        (dwRet & FILE_ATTRIBUTE_DIRECTORY);
#else
    WORD attrib = 0;
    return (0 == _dos_getfileattr( pszPath,&attrib) ) &&
        (attrib & _A_SUBDIR);
#endif
}


static void formatNumber( char *pszNumStr, DWORD dwNum ) 
{
    char szUnformattedNumber[ 32 ];
    sprintf( szUnformattedNumber, "%lu", dwNum );
    
    {
        int n = strlen( szUnformattedNumber );
        int m = (n - 1) / 3;
        int i = 0;
        
        pszNumStr[ m + n ] = 0;
        while ( 0 < n-- ) {
            if ( 3 == i ) {
                pszNumStr[ n + m-- ] = ',';
                i = 0;
            }
            pszNumStr[ n + m ] = szUnformattedNumber[ n ];
            i++;
        }
    }
}


static BOOL isPathSeparator( char ch ) 
{
    return ('\\' == ch) || ('/' == ch);
}


void main( int argc, char** argv ) 
{
    char szDir[ MAX_PATH ];
    
    if ( 1 < argc && 0 != argv[ 1 ][ 0 ] ) {
        strcpy( szDir, argv[ 1 ] );
    } else {
        _getcwd( szDir, MAX_PATH );
    }
    
    if ( !isDirectory( szDir ) ) {
        fprintf( stderr, "%s is not a directory\n", szDir );
        exit( 1 );
    }
    
    {
        int nLen_1 = strlen( szDir ) - 1;
        assert( 0 < nLen_1 );
        if ( isPathSeparator( szDir[ nLen_1 ] ) )  {
            szDir[ nLen_1 ] = 0;
        }
    }
    
    g_dwBytesPerCluster = getClusterSize( szDir );
    
    {
        DWORD dwSize = 0, dwUsed = 0;
        char szSize[ 40 ], szUsed[ 40 ];
        
        getDirectorySize( szDir, &dwSize, &dwUsed );
        
        formatNumber( szSize, dwSize );
        formatNumber( szUsed, dwUsed );
        
        printf("%s bytes (%s bytes used)\n", szSize, szUsed );
    }
}

// end of file

