/*
   Format.dll 
   Helper function to format drives built on the
   top of undocumented Shell32's SHFormatDrive.
*/

#include "format.h"
static BOOL g_bIsNT;               // is NT?  

typedef INT (WINAPI*FORMAT_FUNC)(HWND, INT, INT, INT);

// format a drive calling the standard SHFormatDrive
INT APIENTRY FormatDrive( HWND hWnd, INT iDrive, INT iCapacity,
      INT iType )
{
    HANDLE hLib;
    FORMAT_FUNC pFunc;
    INT irc;
    OSVERSIONINFO os;
    CHAR sz[5];

    // check the underlying OS 
    os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    GetVersionEx( &os );
    g_bIsNT = (os.dwPlatformId==VER_PLATFORM_WIN32_NT);

    // check the drive type (floppy only)
    wsprintf( sz, "%c:\\", 'A'+iDrive );
    if( GetDriveType(sz) != DRIVE_REMOVABLE )
    return ERR_NOTFLOPPY;

    // load the library and get the func address
    hLib = LoadLibrary( "shell32.dll" );
    if( !hLib )
        return ERR_LIBNOTFOUND;
    pFunc = (FORMAT_FUNC)GetProcAddress( hLib, "SHFormatDrive" );
    if( !pFunc )
        return ERR_FUNCNOTFOUND;

    // adjust the capacity param
    if( !g_bIsNT )
      switch( iCapacity )   
      {
        case FORMAT_CAP_360:
          iCapacity = 0x03;
          break;
        case FORMAT_CAP_720:
          iCapacity = 0x05;
          break;
      }
      else
          iCapacity = 0;

    // adjust the format type param
    if( g_bIsNT )
       iType = ((iType==QUICK_FORMAT) ?1 :0);

    // call the function and free the library
    irc = pFunc( hWnd, iDrive, iCapacity, iType );
    FreeLibrary( hLib );

    if( irc==0 && g_bIsNT )
        return ERR_NONE;
    if( irc==0 )
        return ERR_WRONGPARAMS;
    if( irc==-3 )
        return ERR_IMPOSSIBLE;
    if( irc==-2 )
        return ERR_CANCELED;
    if( irc==-1 && g_bIsNT )
        return ERR_CANCELED;
    if( irc==-1 )
        return ERR_IMPOSSIBLE;
    if( irc==6 )
        return ERR_NONE;
    return irc;
}
